package com.example.angel.metrosalud

import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.view.View.GONE
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import com.example.angel.metrosalud.adapters.SafeClickListener
import com.example.angel.metrosalud.data.Achievement
import com.example.angel.metrosalud.data.Alumno
import com.example.angel.metrosalud.data.ParadaWithAchievements
import com.example.angel.metrosalud.data.StravaData
import com.example.angel.metrosalud.databinding.ActivityAchievementBinding
import com.example.angel.metrosalud.myFirebase.MyCallback
import com.example.angel.metrosalud.myFirebase.MyFirebaseAPI
import com.example.angel.metrosalud.myStrava.myStrava
import com.example.angel.metrosalud.myUI.LoadingDialog
import com.fasterxml.jackson.databind.ObjectMapper
import okhttp3.Call
import okhttp3.Callback
import okhttp3.Response
import java.io.IOException
import java.text.SimpleDateFormat
import java.util.*

class AchievementActivity : AppCompatActivity() {

    private lateinit var binding: ActivityAchievementBinding
    private lateinit var alumno: Alumno
    private lateinit var data: ParadaWithAchievements
    private lateinit var dataList: ArrayList<StravaData>
    private  var indexOfParada: Int = -1
    private var totalPoints: Int = 0
    private lateinit var loading: LoadingDialog

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityAchievementBinding.inflate(layoutInflater)
        setContentView(binding.root)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.title = "Parada"
        initialise()
    }


    private fun initialise() {
        loading = LoadingDialog(this)
        dataList = arrayListOf()
        data = intent.getSerializableExtra("item") as ParadaWithAchievements
        alumno = intent. getSerializableExtra("alumno") as Alumno
        indexOfParada = intent.getIntExtra("index", -1)
        totalPoints = intent.getIntExtra("totalPoints",0)
        binding.tvStationName.text = data.name
        binding.tvStationDescription.text = data.description
        binding.tvDate1.text = data.dataIni
        binding.tvDate2.text = data.dataEnd
        binding.tvGoalForGold.text = data.gold
        binding.tvGoldPoints.text = "100"
        binding.tvGoalForSilver.text = data.silver
        binding.tvSilverPoints.text = "50"
        binding.tvGoalForBronze.text = data.bronze
        binding.tvBronzePoints.text = "25"
        when (data.achievementMedal){
            "bronze" -> {
                binding.ivCheckBronze.setImageResource(android.R.drawable.checkbox_on_background)
            }
            "silver" -> {
                binding.ivCheckSilver.setImageResource(android.R.drawable.checkbox_on_background)
            }
            "gold" -> {
                binding.ivCheckGold.setImageResource(android.R.drawable.checkbox_on_background)
                binding.imbBtnStrava.visibility = GONE
            }
        }
        binding.btnLeaderboard.setSafeOnClickListener {
            MyFirebaseAPI(this).getMyGroup(object : MyCallback {
                override fun onGetMyGroup(str: String) {
                    val intent = Intent(this@AchievementActivity, LeaderboardActivity::class.java)
                    intent.putExtra("idGroup", str)
                    startActivity(intent)
                }
            })
        }
        binding.imbBtnStrava.setSafeOnClickListener {
            /**Connect to Strava to get athlete activities*/
            /**First, check if access_token is valid*/
            loading.startLoading()
            //long currentTimeMillis ()-Returns the current time in milliseconds.
            var millis: Long = System.currentTimeMillis()
            //Divide millis by 1000 to get the number of seconds.
            val seconds = millis / 1000
            if (seconds < alumno.expiresAt?.toLong() ?: 0){
                /**Access token is valid*/
                getActivities(alumno.accessToken, data.dataIni, data.dataEnd)
                //checkActivities()
            }else{
                /**We need to refresh access token*/
                myStrava().getAccessToken(alumno.refreshToken.toString(),
                    alumno.accessToken.toString(), object : Callback {
                        override fun onFailure(call: Call, e: IOException) {
                            loading.isDismiss()
                        }

                        override fun onResponse(call: Call, response: Response) {
                            if (response.isSuccessful) {
                                val responseStr: String = response.body?.string() ?: ""
                                val mapperAll = ObjectMapper()
                                val objData = mapperAll.readTree(responseStr)
                                val newAccessToken: String = objData.get("access_token").textValue()
                                val newExpiresAt: String = objData.get("expires_at").longValue()
                                    .toString()
                                /** Update Alumno with new access Token*/
                                alumno.accessToken = newAccessToken
                                alumno.expiresAt = newExpiresAt
                                MyFirebaseAPI(this@AchievementActivity).updateStudent(alumno,
                                    object : MyCallback {
                                        override fun onUpdateStudent(b: Boolean) {
                                            if (b) {
                                                /**Get Athlete activities */
                                                getActivities(alumno.accessToken, data.dataIni,
                                                    data.dataEnd
                                                )
                                                //checkActivities()
                                            } else {
                                                Toast.makeText(this@AchievementActivity,
                                                    "Error actualizando datos del alumno",
                                                    Toast.LENGTH_SHORT)
                                                    .show()
                                                loading.isDismiss()
                                            }
                                        }
                                    })
                                Log.d("STRAVA",
                                    "acces: ${newAccessToken.toString()} y expires: ${newExpiresAt.toString()}"
                                )

                            }
                        }

                    })
            }

        }

    }
    /**Check Activities in dataList array  */
    private fun checkActivities() {
        /**dataList array is ordered from most recent to older, then reverse*/
        dataList.reverse()
        if (data.achievementMedal != "gold"){
            val pattern = "dd/MM/yyyy"
            val simpleDateFormat = SimpleDateFormat(pattern)
            val date = simpleDateFormat.format(Date())
            when (data.type){
                "Kilometros" -> {
                    var medal = dataList.find { a ->
                        a.type.equals("Run") && a.distance!! > data.gold!!.toDouble().times(
                            1000
                        )
                    }
                    if (medal == null) { //Check Gold
                        medal = dataList.find { a ->
                            a.type.equals("Run") && a.distance!! > data.silver!!.toDouble().times(
                                1000
                            )
                        }
                        if (medal == null) { //Check Silver
                            medal = dataList.find { a ->
                                a.type.equals("Run") && a.distance!! > data.bronze!!.toDouble()
                                    .times(1000)
                            }
                            if (medal != null) { //Check Bronze
                                //Is Bronze
                                    if(data.achievementMedal.equals("bronze")){
                                        //Do nothing because is already bronze
                                        showAchievementMessage("no_changes")
                                    }else{
                                        runOnUiThread {
                                            binding.ivCheckBronze.setImageResource(
                                                android.R.drawable.checkbox_on_background
                                            )
                                            //Update Points and hide strava button
                                            val achievement = Achievement(data.name, "bronze", date,"25")
                                            MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                                alumno.idGrupo!!,totalPoints+25, object: MyCallback{
                                                    override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                                        if (b){
                                                            showAchievementMessage("bronze")
                                                        }else{
                                                            loading.isDismiss()
                                                            Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                                    "actualizando sus puntos", Toast.LENGTH_LONG)
                                                                .show()
                                                        }
                                                    }
                                                })
                                            binding.imbBtnStrava.visibility = GONE
                                        }
                                    }
                            }else{
                                //Is nothing
                                showAchievementMessage("nothing")
                            }
                        } else {
                            //Is Silver
                                if (data.achievementMedal.equals("silver")){
                                    //Don nothing because is already silver
                                    showAchievementMessage("no_changes")
                                }else{
                                    runOnUiThread {
                                        binding.ivCheckSilver.setImageResource(
                                            android.R.drawable.checkbox_on_background
                                        )
                                        //Update Points and hide strava button
                                        val achievement = Achievement(data.name, "silver", date,"50")
                                        MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                            alumno.idGrupo!!, totalPoints + 50, object: MyCallback{
                                                override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                                    if (b){
                                                        showAchievementMessage("silver")
                                                    }else{
                                                        loading.isDismiss()
                                                        Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                                "actualizando sus puntos", Toast.LENGTH_LONG)
                                                            .show()
                                                    }
                                                }
                                            })
                                        binding.imbBtnStrava.visibility = GONE
                                    }
                                }
                        }
                    } else {
                        //Is gold
                        runOnUiThread {
                            binding.ivCheckGold.setImageResource(
                                android.R.drawable.checkbox_on_background
                            )
                            //Update Points and hide strava button
                            val achievement = Achievement(data.name, "gold", date,"100")
                            MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                alumno.idGrupo!!, totalPoints + 100, object: MyCallback{
                                    //Show message
                                    override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                        if (b){
                                            showAchievementMessage("gold")
                                        }else{
                                            loading.isDismiss()
                                            Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                    "actualizando sus puntos", Toast.LENGTH_LONG)
                                                .show()
                                        }
                                    }
                                })
                            binding.imbBtnStrava.visibility = GONE
                        }
                    }
                }
                "Desnivel" -> {
                    var medal = dataList.find { a ->
                        a.type.equals("Run") && a.total_elevation_gain!! > data.gold!!.toDouble()
                    }
                    if (medal == null) { //Check Gold
                        medal = dataList.find { a ->
                            a.type.equals("Run"
                            ) && a.total_elevation_gain!! > data.silver!!.toDouble()
                        }
                        if (medal == null) { //Check Silver
                            medal = dataList.find { a ->
                                a.type.equals("Run"
                                ) && a.total_elevation_gain!! > data.bronze!!.toDouble()
                            }
                            if (medal != null) { //Check Bronze
                                //Is Bronze
                                    if (data.achievementMedal.equals("bronze")){
                                        //No changes because is already bronze
                                        showAchievementMessage("no_changes")
                                    }else{
                                        runOnUiThread {
                                            binding.ivCheckBronze.setImageResource(
                                                android.R.drawable.checkbox_on_background
                                            )
                                            //Update Points and hide strava button
                                            val achievement = Achievement(data.name, "bronze", date,"25")
                                            MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                                alumno.idGrupo!!, totalPoints +25, object: MyCallback{
                                                    //Show message
                                                    override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                                        if (b){
                                                            showAchievementMessage("bronze")
                                                        }else{
                                                            loading.isDismiss()
                                                            Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                                    "actualizando sus puntos", Toast.LENGTH_LONG)
                                                                .show()
                                                        }
                                                    }
                                                })
                                            binding.imbBtnStrava.visibility = GONE
                                        }
                                    }
                            }else{
                                showAchievementMessage("nothing")
                            }
                        } else {
                            //Is Silver
                                if (data.achievementMedal.equals("silver")){
                                    //Do nothing because is already silver
                                    showAchievementMessage("no_changes")
                                }else{
                                    runOnUiThread {
                                        binding.ivCheckSilver.setImageResource(
                                            android.R.drawable.checkbox_on_background
                                        )
                                        //Update Points and hide strava button
                                        val achievement = Achievement(data.name, "silver", date,"50")
                                        MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                            alumno.idGrupo!!, totalPoints + 50, object: MyCallback{
                                                //Show message
                                                override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                                    if (b){
                                                        showAchievementMessage("silver")
                                                    }else{
                                                        loading.isDismiss()
                                                        Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                                "actualizando sus puntos", Toast.LENGTH_LONG)
                                                            .show()
                                                    }
                                                }
                                            })
                                        binding.imbBtnStrava.visibility = GONE
                                    }
                                }
                        }
                    } else {
                        //Is gold
                        runOnUiThread {
                            binding.ivCheckGold.setImageResource(
                                android.R.drawable.checkbox_on_background
                            )
                            //Update Points and hide strava button
                            val achievement = Achievement(data.name, "gold", date,"25")
                            MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                alumno.idGrupo!!, totalPoints + 25, object: MyCallback{
                                    //Show message
                                    override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                        if (b){
                                            showAchievementMessage("gold")
                                        }else{
                                            loading.isDismiss()
                                            Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                    "actualizando sus puntos", Toast.LENGTH_LONG)
                                                .show()
                                        }
                                    }
                                })
                            binding.imbBtnStrava.visibility = GONE
                        }
                    }

                }
                "Desnivel total" -> {
                    var differenceInLevels: ArrayList<Double> = dataList.map { a -> a.total_elevation_gain } as ArrayList<Double>
                    var totalDifferenceInLevels = differenceInLevels.reduce { acc, d -> acc + d }
                    if (totalDifferenceInLevels > data.gold!!.toLong()) {
                        //Is Gold
                        runOnUiThread {
                            binding.ivCheckGold.setImageResource(
                                android.R.drawable.checkbox_on_background
                            )
                            //Update Points and hide strava button
                            val achievement = Achievement(data.name, "gold", date,"100")
                            MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                alumno.idGrupo!!,totalPoints + 100, object: MyCallback{
                                    //Show message
                                    override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                        if (b){
                                            showAchievementMessage("gold")
                                        }else{
                                            loading.isDismiss()
                                            Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                    "actualizando sus puntos", Toast.LENGTH_LONG)
                                                .show()
                                        }
                                    }
                                })
                            binding.imbBtnStrava.visibility = GONE
                        }

                    } else {
                        if (totalDifferenceInLevels > data.silver!!.toLong()) {
                            //Is silver
                                if (data.achievementMedal.equals("silver")){
                                    //Do nothing because is already silver
                                    showAchievementMessage("no_changes")
                                }else{
                                    runOnUiThread {
                                        binding.ivCheckSilver.setImageResource(
                                            android.R.drawable.checkbox_on_background
                                        )
                                        //Update Points and hide strava button
                                        val achievement = Achievement(data.name, "silver", date,"50")
                                        MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                            alumno.idGrupo!!,totalPoints + 50, object: MyCallback{
                                                //Show message
                                                override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                                    if (b){
                                                        showAchievementMessage("silver")
                                                    }else{
                                                        loading.isDismiss()
                                                        Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                                "actualizando sus puntos", Toast.LENGTH_LONG)
                                                            .show()
                                                    }
                                                }
                                            })
                                        binding.imbBtnStrava.visibility = GONE
                                    }
                                }
                        } else {
                            if (totalDifferenceInLevels > data.bronze!!.toLong()) {
                                //Is Bronze
                                    if (data.achievementMedal.equals("bronze")){
                                        //Don nothing because is already bronze
                                    }else{
                                        runOnUiThread {
                                            binding.ivCheckBronze.setImageResource(
                                                android.R.drawable.checkbox_on_background
                                            )
                                            //Update Points and hide strava button
                                            val achievement = Achievement(data.name, "bronze", date,"25")
                                            MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                                alumno.idGrupo!!,totalPoints + 25, object: MyCallback{
                                                    //Show message
                                                    override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                                        if (b){
                                                            showAchievementMessage("bronze")
                                                        }else{
                                                            loading.isDismiss()
                                                            Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                                    "actualizando sus puntos", Toast.LENGTH_LONG)
                                                                .show()
                                                        }
                                                    }
                                                })
                                            binding.imbBtnStrava.visibility = GONE
                                        }
                                    }
                            } else {
                                // in nothing
                                showAchievementMessage("nothing")
                            }
                        }
                    }

                }
                "Minutos total" -> {
                    var minutes: ArrayList<Double> = dataList.map { a ->
                        a.moving_time?.div(60)
                    } as ArrayList<Double>
                    var totalMinutes = minutes.reduce { acc, d -> acc + d }
                    if (totalMinutes > data.gold!!.toLong()) {
                        //Is Gold
                        runOnUiThread {
                            binding.ivCheckGold.setImageResource(
                                android.R.drawable.checkbox_on_background
                            )
                            //Update Points and hide strava button
                            val achievement = Achievement(data.name, "gold", date,"100")
                            MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                alumno.idGrupo!!,totalPoints + 100, object: MyCallback{
                                    //Show message
                                    override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                        if (b){
                                            showAchievementMessage("gold")
                                        }else{
                                            loading.isDismiss()
                                            Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                    "actualizando sus puntos", Toast.LENGTH_LONG)
                                                .show()
                                        }
                                    }
                                })
                            binding.imbBtnStrava.visibility = GONE
                        }

                    } else {
                        if (totalMinutes > data.silver!!.toLong()) {
                            //Is silver
                                if(data.achievementMedal.equals("silver")){
                                    //Do nothing because is already silver
                                    showAchievementMessage("no_changes")
                                }else{
                                    runOnUiThread {
                                        binding.ivCheckSilver.setImageResource(
                                            android.R.drawable.checkbox_on_background
                                        )
                                        //Update Points and hide strava button
                                        val achievement = Achievement(data.name, "silver", date,"50")
                                        MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                            alumno.idGrupo!!,totalPoints + 50, object: MyCallback{
                                                //Show message
                                                override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                                    if (b){
                                                        showAchievementMessage("silver")
                                                    }else{
                                                        loading.isDismiss()
                                                        Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                                "actualizando sus puntos", Toast.LENGTH_LONG)
                                                            .show()
                                                    }
                                                }
                                            })
                                        binding.imbBtnStrava.visibility = GONE
                                    }
                                }
                        } else {
                            if (totalMinutes > data.bronze!!.toLong()) {
                                //Is Bronze
                                    if(data.achievementMedal.equals("bronze")){
                                        //Do nothing because is already bronze
                                        showAchievementMessage("no_changes")
                                    }else{
                                        runOnUiThread {
                                            binding.ivCheckBronze.setImageResource(
                                                android.R.drawable.checkbox_on_background
                                            )
                                            //Update Points and hide strava button
                                            val achievement = Achievement(data.name, "bronze", date,"25")
                                            MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                                alumno.idGrupo!!,totalPoints + 25, object: MyCallback{
                                                    //Show message
                                                    override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                                        if (b){
                                                            showAchievementMessage("bronze")
                                                        }else{
                                                            loading.isDismiss()
                                                            Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                                    "actualizando sus puntos", Toast.LENGTH_LONG)
                                                                .show()
                                                        }
                                                    }
                                                })
                                            binding.imbBtnStrava.visibility = GONE
                                        }
                                    }
                            } else {
                                // in nothing
                                showAchievementMessage("nothing")
                            }
                        }
                    }
                }
                "Kilometros total" -> {
                    var km: ArrayList<Double> = dataList.map { a ->
                        a.distance?.div(1000)
                    } as ArrayList<Double>
                    var totalKm = km.reduce { acc, d -> acc + d }
                    if (totalKm > data.gold!!.toLong()) {
                        //Is Gold
                        runOnUiThread {
                            binding.ivCheckGold.setImageResource(
                                android.R.drawable.checkbox_on_background
                            )
                            //Update Points and hide strava button
                            val achievement = Achievement(data.name, "gold", date,"100")
                            MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                alumno.idGrupo!!,totalPoints + 100, object: MyCallback{
                                    //Show message
                                    override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                        if (b){
                                            showAchievementMessage("gold")
                                        }else{
                                            loading.isDismiss()
                                            Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                    "actualizando sus puntos", Toast.LENGTH_LONG)
                                                .show()
                                        }
                                    }
                                })
                            binding.imbBtnStrava.visibility = GONE
                        }

                    } else {
                        if (totalKm > data.silver!!.toLong()) {
                            //Is silver
                                if(data.achievementMedal.equals("silver")){
                                    //Do nothing because is already silver
                                    showAchievementMessage("no_changes")
                                }else{
                                    runOnUiThread {
                                        binding.ivCheckSilver.setImageResource(
                                            android.R.drawable.checkbox_on_background
                                        )
                                        //Update Points and hide strava button
                                        val achievement = Achievement(data.name, "silver", date,"50")
                                        MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                            alumno.idGrupo!!,totalPoints + 50, object: MyCallback{
                                                //Show message
                                                override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                                    if (b){
                                                        showAchievementMessage("silver")
                                                    }else{
                                                        loading.isDismiss()
                                                        Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                                "actualizando sus puntos", Toast.LENGTH_LONG)
                                                            .show()
                                                    }
                                                }
                                            })
                                        binding.imbBtnStrava.visibility = GONE
                                    }
                                }
                        } else {
                            if (totalKm > data.bronze!!.toLong()) {
                                //Is Bronze
                                    if(data.achievementMedal.equals("bronze")){
                                        //Do nothing because is already bronze
                                        showAchievementMessage("no_changes")
                                    }else{
                                        runOnUiThread {
                                            binding.ivCheckBronze.setImageResource(
                                                android.R.drawable.checkbox_on_background
                                            )
                                            //Update Points and hide strava button
                                            val achievement = Achievement(data.name, "bronze", date,"25")
                                            MyFirebaseAPI(this).updatePoints(achievement, indexOfParada,
                                                alumno.idGrupo!!,totalPoints + 25, object: MyCallback{
                                                    //Show message
                                                    override fun onUpdatePointsAndTotalPoints(b: Boolean) {
                                                        if (b){
                                                            showAchievementMessage("bronze")
                                                        }else{
                                                            loading.isDismiss()
                                                            Toast.makeText(this@AchievementActivity, "Se ha producido un error " +
                                                                    "actualizando sus puntos", Toast.LENGTH_LONG)
                                                                .show()
                                                        }
                                                    }
                                                })
                                            binding.imbBtnStrava.visibility = GONE
                                        }
                                    }
                            } else {
                                // Is nothing
                                showAchievementMessage("nothing")
                            }
                        }
                    }
                }

            }
        }
    }
    /** Get student activities from initial date to final date
     * Input: access token, initial date, final date
     * Output: global variable dataList with relevant information
     * of each activity*/
    private fun getActivities(accessToken: String?, dataIni: String?, dataEnd: String?){
        if (accessToken != null) {
            myStrava().getAthleteActivities(accessToken, dataIni, dataEnd, object : Callback {
                override fun onFailure(call: Call, e: IOException) {
                    Log.d("STRAVA", call.toString())
                }
                override fun onResponse(call: Call, response: Response) {
                    if (response.isSuccessful) {
                        /**Process JSON response with Jackson*/
                        val responseStr: String = response.body?.string() ?: ""
                        val mapperAll = ObjectMapper()
                        val objData = mapperAll.readTree(responseStr)
                        objData.toList().forEach { a ->
                            /**Add to data list only activities after initial date */
                            val formatter1 = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'")
                            val formatter2 = SimpleDateFormat("dd/MM/yyyy")
                            val time1 = formatter1.parse(a.get("start_date").textValue()).time / 1000
                            val time2 = formatter2.parse(dataIni).time / 1000
                            if (time1 > time2) {
                                dataList.add(StravaData(
                                    a.get("distance").doubleValue(),
                                    a.get("moving_time").intValue(),
                                    a.get("total_elevation_gain").doubleValue(),
                                    a.get("type").textValue(),
                                    a.get("start_date").textValue(),
                                    a.get("average_speed").doubleValue(),
                                    a.get("max_speed").doubleValue(),
                                    a.get("elev_high").doubleValue(),
                                    a.get("elev_low").doubleValue())
                                )
                            } else {
                                /**If one activity is out of date range, then exit function */
                                checkActivities()
                                return
                            }
                        }
                        /**If all activities added to list, then call again getActivities with new end date*/
                        val dataListSize = dataList.size
                        val newData: String? = dataList[dataListSize - 1].start_date
                        val newEndData = newData?.substring(8, 10) + "/" + newData?.substring(5, 7) + "/" + newData?.substring(0, 4)
                        getActivities(accessToken, dataIni, newEndData)
                    }
                }
            })
        }
    }

    private fun showAchievementMessage(medal: String){

        loading.isDismiss() // First, dismiss loading dialog
        runOnUiThread {
            binding.imbBtnStrava.visibility = GONE  // It has no sense to connect again with strava, so better hide it
        }

        var message: String = ""
        var title: String = getString(R.string.dialog_achievement_title)
        when (medal){
            "gold" -> {
                message = getString(R.string.dialog_achievement_gold_message)

            }
            "silver" -> {
                message = getString(R.string.dialog_achievement_silver_message)
            }
            "bronze" -> {
                message = getString(R.string.dialog_achievement_bronze_message)
            }
            "nothing" -> {
                message = getString(R.string.dialog_no_achievement_message)
                title = getString(R.string.dialog_no_achievement_title)
            }
            "no_changes" -> {
                message = getString(R.string.dialog_no_changes_achievement_message)
                title = getString(R.string.dialog_no_changes_achievement_title)
            }
        }
        runOnUiThread{
            val dialog = AlertDialog.Builder(this)
                .setTitle(title)
                .setMessage(message)
                .setNeutralButton(R.string.dialog_achievement_aceptar){ view, _ ->
                    view.dismiss()
                }
                .setCancelable(false)
                .create()
            dialog.show()
        }

    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when(item.itemId){
            R.id.exit -> {
                val intent = Intent(Intent.ACTION_MAIN)
                intent.addCategory(Intent.CATEGORY_HOME)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                startActivity(intent)

            }
            R.id.logOut -> {
                MyFirebaseAPI(this).logOut()
                val intent = Intent(this, MainActivity::class.java)
                finish()
                startActivity(intent)
            }
            android.R.id.home -> {
                onBackPressed()
            }
        }
        return super.onOptionsItemSelected(item)
    }
    private fun View.setSafeOnClickListener(onSafeClick: (View) -> Unit) {
        val safeClickListener = SafeClickListener {
            onSafeClick(it)
        }
        setOnClickListener(safeClickListener)
    }
}