package com.example.angel.metrosalud

import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.recyclerview.widget.LinearLayoutManager
import com.example.angel.metrosalud.adapters.SafeClickListener
import com.example.angel.metrosalud.adapters.StationAdapter
import com.example.angel.metrosalud.data.Group
import com.example.angel.metrosalud.data.Parada
import com.example.angel.metrosalud.databinding.ActivityCreateGroupBinding
import com.example.angel.metrosalud.myFirebase.MyCallback
import com.example.angel.metrosalud.myFirebase.MyFirebaseAPI
import com.example.angel.metrosalud.myUI.LoadingDialog
import java.text.SimpleDateFormat
import java.util.*

class CreateGroupActivity : AppCompatActivity(), StationAdapter.OnItemClickListener {

    private lateinit var binding: ActivityCreateGroupBinding
    private lateinit var loading: LoadingDialog

    /**Default Stations when adding a group*/
    private var stationList: MutableList<Parada> = mutableListOf(
        Parada("Sube una montaña muy alta", "Acumula todo el desnivel que puedas en una misma actividad",
            "Desnivel", "150", "300", "750", "100", getIniDate(), getEndDate()
        ),
        Parada("Salimos a correr", "Corre la mayor distancia que puedas en una actividad", "Kilometros",
            "25", "50", "100", "100", getIniDate(), getEndDate()
        ),
        Parada("Nos vamos a la naturaleza", "Sube montañas durante este mes y acumula todo el desnivel que puedas",
            "Desnivel total", "250", "500", "1500", "100", getIniDate(), getEndDate()
        ),
        Parada("Pasar a la acción", "Acumula minutos de actividad durante este mes", "Minutos total",
            "100", "200", "300", "100", getIniDate(), getEndDate()
        ),
        )

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityCreateGroupBinding.inflate(layoutInflater)
        loading = LoadingDialog(this)
        setContentView(binding.root)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.title = "Crear Grupo"
        initRecycler()
        initialise()
    }

    private fun initRecycler() {
        binding.rvStation.layoutManager = LinearLayoutManager(this)
        val adapter = StationAdapter(stationList, this)
        binding.rvStation.adapter = adapter
    }

    private fun initialise(){
        binding.btnAddStation.setSafeOnClickListener {
            val intent = Intent(this, CreateStationActivity::class.java)
            startActivityForResult(intent, 2)
        }
        binding.btnAddGroup.setSafeOnClickListener {
            if(checkForm()){
                /**Save Group at Firebase database and go back to Teacher Screen*/
                if (binding.etNewGroup.text.isNotEmpty()){
                    val myGroup = Group(null, binding.etNewGroup.text.toString(),null,null,stationList)
                    /**First get maxInscription code to add 1
                     * Very simple way to generate code */
                    loading.startLoading()
                    MyFirebaseAPI(this).getMaxInscriptionCode(object: MyCallback{
                        override fun onGetMaxInscriptionCode(id: String) {
                            myGroup.inscriptionCode = (id.toInt()+1).toString() /**Update inscriptionCode with new calculated one*/
                            MyFirebaseAPI(this@CreateGroupActivity).addGroup(myGroup, object : MyCallback{
                                override fun onAddGroup(b: Boolean) {
                                    loading.isDismiss()
                                    if (b){
                                        val dialog = AlertDialog.Builder(this@CreateGroupActivity)
                                            .setTitle(R.string.dialog_inscription_code_title)
                                            .setMessage(getString(R.string.dialog_inscription_code_description)+myGroup.inscriptionCode)
                                            .setNeutralButton(R.string.dialog_aceptar){ view, _ ->
                                                view.dismiss()
                                                val intent = Intent(this@CreateGroupActivity, TeacherInitActivity::class.java)
                                                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
                                                finish()
                                                startActivity(intent)
                                            }
                                            .setCancelable(false)
                                            .create()
                                        dialog.show()
                                        /*val intent = Intent(this@CreateGroupActivity, TeacherInitActivity::class.java)
                                        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
                                        finish()
                                        startActivity(intent)*/
                                    }else{
                                        Toast.makeText(this@CreateGroupActivity, "Error añadiendo grupo", Toast.LENGTH_SHORT)
                                            .show()
                                    }
                                }
                            })
                        }
                    })

                }else{
                    Toast.makeText(this@CreateGroupActivity, "Debes darle un nombre al grupo", Toast.LENGTH_SHORT)
                        .show()
                }
            }else{
                Toast.makeText(this@CreateGroupActivity, "Debes darle un nombre al grupo e incluir al menos 4 estaciones", Toast.LENGTH_LONG)
                    .show()
            }
        }
    }

    private fun checkForm(): Boolean{
        //Check if name is not empty and if there are at least 4 stations
        return binding.etNewGroup.text.isNotEmpty() && stationList.size > 3

    }

    override fun onRemoveClick(position: Int) {
        stationList.removeAt(position)
        binding.rvStation.adapter?.notifyItemRemoved(position)
    }

    override fun onEditClick(position: Int) {
        val myStation: Parada = stationList.get(position)
        val intent = Intent(this, EditStationActivity::class.java)
        intent.putExtra("station", myStation)
        intent.putExtra("index", position)
        startActivityForResult(intent, 1)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        // From Edit Station Screen
        if (requestCode == 1) {
            if (resultCode == RESULT_OK) run {
                val stationResult: Parada = data?.getSerializableExtra("result") as Parada
                val indexResult: Int = data.getIntExtra("index", 0)
                stationList[indexResult] = stationResult
                binding.rvStation.adapter?.notifyDataSetChanged()
            }
            //From Add Station Screen
        } else if (requestCode == 2) {
            if (resultCode == RESULT_OK) run {
                val stationResult: Parada = data?.getSerializableExtra("result") as Parada
                stationList.add(stationResult)
                binding.rvStation.adapter?.notifyDataSetChanged()
            }

        }
    }

    /**Function that returns default start date for stations*/
    private fun getIniDate(): String {
        val simpleDateFormat = SimpleDateFormat("dd/MM/yyyy")
        return simpleDateFormat.format(Date())
    }

    /**Function that returns default end date for stations*/
    private fun getEndDate(): String {
        val date = Calendar.getInstance()
        date.add(Calendar.MONTH, 1)
        val simpleDateFormat = SimpleDateFormat("dd/MM/yyyy")
        return simpleDateFormat.format(date.time)
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when(item.itemId){
            R.id.exit -> {
                val intent = Intent(Intent.ACTION_MAIN)
                intent.addCategory(Intent.CATEGORY_HOME)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                startActivity(intent)

            }
            R.id.logOut -> {
                MyFirebaseAPI(this).logOut()
                val intent = Intent(this, MainActivity::class.java)
                finish()
                startActivity(intent)
            }
            android.R.id.home -> {
                onBackPressed()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    private fun View.setSafeOnClickListener(onSafeClick: (View) -> Unit) {
        val safeClickListener = SafeClickListener {
            onSafeClick(it)
        }
        setOnClickListener(safeClickListener)
    }
}