package com.example.angel.metrosalud

import android.app.DatePickerDialog
import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.Spinner
import android.widget.Toast
import com.example.angel.metrosalud.adapters.SafeClickListener
import com.example.angel.metrosalud.data.Parada
import com.example.angel.metrosalud.myFirebase.MyFirebaseAPI
import com.example.angel.metrosalud.myUI.DatePickerFragment
import java.text.SimpleDateFormat
import java.util.*

class CreateStationActivity : AppCompatActivity() {

    private lateinit var spinner: Spinner
    private lateinit var etStationName: EditText
    private lateinit var etStationDescription: EditText
    private lateinit var etBronze: EditText
    private lateinit var etSilver: EditText
    private lateinit var etGold: EditText
    private lateinit var etPoints: EditText
    private lateinit var etIniDate: EditText
    private lateinit var etEndDate: EditText
    private lateinit var btnAddStation: Button
    private lateinit var btnBack: Button

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_create_station)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.title = "Crear estación"
        initialise()
    }

    private fun initialise() {
        // Data binding
        spinner = findViewById(R.id.spinnerStationType)
        etStationName = findViewById(R.id.etStationName)
        etStationDescription = findViewById(R.id.etStationDescription)
        etBronze = findViewById(R.id.etBronze)
        etSilver = findViewById(R.id.etSilver)
        etGold = findViewById(R.id.etGold)
        etPoints = findViewById(R.id.etPoints)
        etIniDate = findViewById(R.id.etIniDate)
        etEndDate = findViewById(R.id.etEndDate)
        btnAddStation = findViewById(R.id.btnAddStation)
        btnBack = findViewById(R.id.btnBackStation)
        etIniDate.setText(getIniDate())
        etIniDate.setSafeOnClickListener {
            showDatePickerDialog(etIniDate)
        }
        etEndDate.setText(getEndDate())
        etEndDate.setSafeOnClickListener {
            showDatePickerDialog(etEndDate)
        }

        btnAddStation.setSafeOnClickListener {
            if (checkForm()) {
                val stationResult: Parada = getStation()
                val resultIntent = Intent(this, CreateGroupActivity::class.java)
                resultIntent.putExtra("result", stationResult)
                setResult(RESULT_OK, resultIntent)
                finish()
            }
        }

        btnBack.setSafeOnClickListener {
            finish() }
    }

    private fun showDatePickerDialog(etDate: EditText) {
        val newFragment = DatePickerFragment.newInstance { _, year, month, day ->
            val dayStr = day.twoDigits()
            val monthStr = (month + 1).twoDigits() // +1 because January is zero

            val selectedDate = "$dayStr/$monthStr/$year"
            etDate.setText(selectedDate)
        }

        newFragment.show(supportFragmentManager, "datePicker")
    }

    private fun Int.twoDigits() =
            if (this <= 9) "0$this" else this.toString()

    //Function that checks Form Station is valid
    private fun checkForm(): Boolean {
        if(etStationName.text.isNotEmpty() && etStationDescription.text.isNotEmpty() && etBronze.text.isNotEmpty()
                && etSilver.text.isNotEmpty() && etGold.text.isNotEmpty() && etPoints.text.isNotEmpty() && etEndDate.text.isNotEmpty()
                && etIniDate.text.isNotEmpty()){
                    if(etGold.text.toString().toInt() > etSilver.text.toString().toInt() &&
                            etSilver.text.toString().toInt() > etBronze.text.toString().toInt()){
                        return true
                    }else{
                        Toast.makeText(this, "El objetivo para oro debe ser mayor que el objetivo para plata." +
                                " Y el de plata mayor que el de bronce", Toast.LENGTH_LONG)
                            .show()
                        return false
                    }

        }else{
            Toast.makeText(this, "Debe rellenar todos los campos", Toast.LENGTH_LONG)
                .show()
            return false
        }
    }


    //Function that returns default start date for stations
    private fun getIniDate(): String {
        val simpleDateFormat = SimpleDateFormat("dd/MM/yyyy")
        return simpleDateFormat.format(Date())
    }

    //Function that returns default end date for stations
    private fun getEndDate(): String {
        val date = Calendar.getInstance()
        date.add(Calendar.MONTH, 1)
        val simpleDateFormat = SimpleDateFormat("dd/MM/yyyy")
        return simpleDateFormat.format(date.time)
    }

    private fun getStation(): Parada {
        val myStation = Parada(
            etStationName.text.toString(),
            etStationDescription.text.toString(),
            spinner.selectedItem.toString(),
            etBronze.text.toString(),
            etSilver.text.toString(),
            etGold.text.toString(),
            etPoints.text.toString(),
            etIniDate.text.toString(),
            etEndDate.text.toString()
        )
        Log.d("STATION", myStation.toString())
        return myStation

    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when(item.itemId){
            R.id.exit -> {
                val intent = Intent(Intent.ACTION_MAIN)
                intent.addCategory(Intent.CATEGORY_HOME)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                startActivity(intent)

            }
            R.id.logOut -> {
                MyFirebaseAPI(this).logOut()
                val intent = Intent(this, MainActivity::class.java)
                finish()
                startActivity(intent)
            }
            android.R.id.home -> {
                onBackPressed()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    private fun View.setSafeOnClickListener(onSafeClick: (View) -> Unit) {
        val safeClickListener = SafeClickListener {
            onSafeClick(it)
        }
        setOnClickListener(safeClickListener)
    }
}