package com.example.angel.metrosalud

import android.app.DatePickerDialog
import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.Spinner
import android.widget.Toast
import com.example.angel.metrosalud.adapters.SafeClickListener
import com.example.angel.metrosalud.data.Parada
import com.example.angel.metrosalud.myFirebase.MyFirebaseAPI
import com.example.angel.metrosalud.myUI.DatePickerFragment


class EditStationActivity : AppCompatActivity() {

    private lateinit var spinner: Spinner
    private lateinit var etStationName: EditText
    private lateinit var etStationDescription: EditText
    private lateinit var etBronze: EditText
    private lateinit var etSilver: EditText
    private lateinit var etGold: EditText
    private lateinit var etPoints: EditText
    private lateinit var etIniDate: EditText
    private lateinit var etEndDate: EditText
    private lateinit var btnSaveChanges: Button
    private lateinit var btnBack: Button

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_edit_station)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.title = "Editar estación"
        initialise()
    }

    private fun initialise() {
        val station: Parada = intent.getSerializableExtra("station") as Parada
        val index: Int = intent.getIntExtra("index", 0)

        // Data binding
        spinner = findViewById(R.id.spinnerStationType)
        etStationName = findViewById(R.id.etStationName)
        etStationDescription = findViewById(R.id.etStationDescription)
        etBronze = findViewById(R.id.etBronze)
        etSilver = findViewById(R.id.etSilver)
        etGold = findViewById(R.id.etGold)
        etPoints = findViewById(R.id.etPoints)
        etIniDate = findViewById(R.id.etIniDate)
        etEndDate = findViewById(R.id.etEndDate)
        btnSaveChanges = findViewById(R.id.btnSaveChangesStation)
        btnBack = findViewById(R.id.btnBackStation)

        etStationName.setText(station.name)
        etStationDescription.setText(station.description)
        etBronze.setText(station.bronze.toString())
        etSilver.setText(station.silver.toString())
        etGold.setText(station.gold.toString())
        etPoints.setText(station.points.toString())
        etIniDate.setText(station.dataIni)
        etIniDate.setSafeOnClickListener {
            showDatePickerDialog(etIniDate)
        }
        etEndDate.setText(station.dataEnd)
        etEndDate.setSafeOnClickListener {
            showDatePickerDialog(etEndDate)
        }
        var type: Int = 0
        when (station.type) {
            "Desnivel" -> type = 0
            "Kilometros" -> type = 1
            "Desnivel total" -> type = 2
            "Minutos total" -> type = 3
            else -> type = 0
        }

        spinner.setSelection(type)
        btnSaveChanges.setSafeOnClickListener {
            if (checkForm()) {
                val stationResult: Parada = getStation()
                val resultIntent = Intent(this, CreateGroupActivity::class.java)
                resultIntent.putExtra("result", stationResult)
                resultIntent.putExtra("index", index)
                setResult(RESULT_OK, resultIntent)
                finish()
            }
        }
        btnBack.setSafeOnClickListener {
            finish()
        }
    }

    private fun showDatePickerDialog(etDate: EditText) {
        val newFragment = DatePickerFragment.newInstance { _, year, month, day ->
            val dayStr = day.twoDigits()
            val monthStr = (month + 1).twoDigits() // +1 because January is zero

            val selectedDate = "$dayStr/$monthStr/$year"
            etDate.setText(selectedDate)
        }

        newFragment.show(supportFragmentManager, "datePicker")
    }

    private fun Int.twoDigits() =
            if (this <= 9) "0$this" else this.toString()

    private fun checkForm(): Boolean {
        // check Form is valid
        if(etStationName.text.isNotEmpty() && etStationDescription.text.isNotEmpty() && etBronze.text.isNotEmpty()
                && etSilver.text.isNotEmpty() && etGold.text.isNotEmpty() && etPoints.text.isNotEmpty() && etEndDate.text.isNotEmpty()
                && etIniDate.text.isNotEmpty()){
            if(etGold.text.toString().toInt() > etSilver.text.toString().toInt() &&
                    etSilver.text.toString().toInt() > etBronze.text.toString().toInt()){
                return true
            }else{
                Toast.makeText(this, "El objetivo para oro debe ser mayor que el objetivo para plata." +
                        " Y el de plata mayor que el de bronce", Toast.LENGTH_LONG)
                    .show()
                return false
            }
        }else{
            Toast.makeText(this, "Debe rellenar todos los campos", Toast.LENGTH_LONG)
                .show()
            return false
        }
    }

    private fun getStation(): Parada {
        val myStation:Parada = Parada(etStationName.text.toString(), etStationDescription.text.toString(), spinner.selectedItem.toString(), etBronze.text.toString(), etSilver.text.toString(), etGold.text.toString(), etPoints.text.toString(), etIniDate.text.toString(), etEndDate.text.toString())
        Log.d("STATION",myStation.toString())
        return myStation
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when(item.itemId){
            R.id.exit -> {
                val intent = Intent(Intent.ACTION_MAIN)
                intent.addCategory(Intent.CATEGORY_HOME)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                startActivity(intent)

            }
            R.id.logOut -> {
                MyFirebaseAPI(this).logOut()
                val intent = Intent(this, MainActivity::class.java)
                finish()
                startActivity(intent)
            }
            android.R.id.home -> {
                onBackPressed()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    private fun View.setSafeOnClickListener(onSafeClick: (View) -> Unit) {
        val safeClickListener = SafeClickListener {
            onSafeClick(it)
        }
        setOnClickListener(safeClickListener)
    }
}