package com.example.angel.metrosalud

import android.content.Intent
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import com.example.angel.metrosalud.adapters.StudentAdapter
import com.example.angel.metrosalud.data.Alumno
import com.example.angel.metrosalud.databinding.ActivityLeaderboardBinding
import com.example.angel.metrosalud.myFirebase.MyCallback
import com.example.angel.metrosalud.myFirebase.MyFirebaseAPI
import java.util.*
import kotlin.Comparator
import kotlin.collections.ArrayList

class LeaderboardActivity : AppCompatActivity(), StudentAdapter.OnItemClickListener {
    private lateinit var binding: ActivityLeaderboardBinding
    private lateinit var studentList: ArrayList<Alumno>

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityLeaderboardBinding.inflate(layoutInflater)
        setContentView(binding.root)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.title = "Clasificación"
        studentList = arrayListOf()
        val idGroup: String? = intent.extras?.getString("idGroup")
        if (idGroup != null) {
            MyFirebaseAPI(this).getStudents(idGroup, object : MyCallback {
                override fun onGetStudents(list: ArrayList<Alumno>) {
                    /**Order list by points */
                    val compareByPoints: Comparator<Alumno> = Comparator { a1: Alumno, a2: Alumno ->
                        a2.puntos!!.toInt().compareTo(a1.puntos!!.toInt())
                    }
                    Collections.sort(list, compareByPoints)
                    studentList.addAll(list.toList())
                    initRecyclerView()
                }
            })
        }
    }

    private fun initRecyclerView() {
        val llm = LinearLayoutManager(this)
        binding.rvLeaderboard.layoutManager = llm
        val adapter = StudentAdapter(studentList, this)
        binding.rvLeaderboard.adapter = adapter
        val dividerDecoration = DividerItemDecoration(this, llm.orientation)
        binding.rvLeaderboard.addItemDecoration(dividerDecoration)
    }



    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when(item.itemId){
            R.id.exit -> {
                val intent = Intent(Intent.ACTION_MAIN)
                intent.addCategory(Intent.CATEGORY_HOME)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                startActivity(intent)

            }
            R.id.logOut -> {
                MyFirebaseAPI(this).logOut()
                val intent = Intent(this, MainActivity::class.java)
                finish()
                startActivity(intent)
            }
            android.R.id.home -> {
                onBackPressed()
            }
        }
        return super.onOptionsItemSelected(item)
    }
}