package com.example.angel.metrosalud

import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.view.Gravity
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.view.View.GONE
import android.view.View.VISIBLE
import android.widget.*
import androidx.appcompat.app.AppCompatActivity
import androidx.core.graphics.toColorInt
import com.example.angel.metrosalud.data.Alumno
import com.example.angel.metrosalud.data.Profesor
import com.example.angel.metrosalud.myFirebase.MyCallback
import com.example.angel.metrosalud.myFirebase.MyFirebaseAPI
import com.example.angel.metrosalud.myUI.LoadingDialog
import com.example.angel.metrosalud.myStrava.myStrava
import com.fasterxml.jackson.databind.ObjectMapper
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.database.DatabaseReference
import com.google.firebase.database.FirebaseDatabase
import com.squareup.picasso.Picasso
import jp.wasabeef.picasso.transformations.CropCircleTransformation
import okhttp3.*
import java.io.IOException


class RegisterActivity : AppCompatActivity() {
    /**UI*/
    private lateinit var spinner: Spinner
    private lateinit var etName: EditText
    private lateinit var etSurname1: EditText
    private lateinit var etSurname2: EditText
    private lateinit var etEmail: EditText
    private lateinit var etPassword: EditText
    private lateinit var buttonRegister: Button
    private lateinit var buttonStrava: ImageButton
    private lateinit var ivPhoto: ImageView
    private lateinit var tvOptional: TextView

    private lateinit var typeUser: String
    private lateinit var loading: LoadingDialog

    /**Firebase*/
    private lateinit var databaseReference: DatabaseReference
    private lateinit var database: FirebaseDatabase
    private lateinit var auth: FirebaseAuth

    /**Strava variables from authorization process with Strava API */
    private var authorizationToken: String? = null
    private var accessToken: String? = null
    private var refreshToken: String? = null
    private var urlPhoto: String? = null
    private var expiresAt: String? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_register)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.title = "Registro"
        initialise()
    }

    private fun initialise() {
        database = FirebaseDatabase.getInstance()
        auth = FirebaseAuth.getInstance()
        databaseReference = database.reference

        etName = findViewById(R.id.editTextRegisterPersonName)
        etSurname1 = findViewById(R.id.editTextRegisterPersonSurname1)
        etSurname2 = findViewById(R.id.editTextRegisterPersonSurname2)
        etEmail = findViewById(R.id.editTextRegisterEmailAddress)
        etPassword = findViewById(R.id.editTextRegisterPassword)
        buttonRegister = findViewById(R.id.buttonRegisterConfirm)
        buttonStrava = findViewById(R.id.buttonStrava)
        spinner = findViewById(R.id.spinnerUserType)
        ivPhoto = findViewById(R.id.ivPlaceholder)
        tvOptional = findViewById(R.id.textViewOptional)


        spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(
                    parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                if (parent != null) {
                    val tv: TextView = parent.getChildAt(0) as TextView
                    tv.gravity = Gravity.CENTER
                }
                if (position == 0) {
                    typeUser = "Profesor"
                    buttonStrava.visibility = GONE
                    ivPhoto.visibility = GONE
                    tvOptional.visibility = GONE
                } else {
                    typeUser = "Alumno"
                    buttonStrava.visibility = VISIBLE
                    ivPhoto.visibility = VISIBLE
                    tvOptional.visibility = VISIBLE
                }
            }

            override fun onNothingSelected(parent: AdapterView<*>?) {
                typeUser = "Undefined"
            }
        }

        buttonRegister.setOnClickListener {
            register()
        }

        buttonStrava.setOnClickListener {
            val intent = Intent(this, AuthorizeStravaActivity::class.java)
            startActivityForResult(intent, 1)
        }
    }

    private fun register() {
        val name: String = etName.text.toString()
        val surname1: String = etSurname1.text.toString()
        val surname2: String = etSurname2.text.toString()
        val email: String = etEmail.text.toString()
        val password: String = etPassword.text.toString()

        /**Check all fields are not empty*/
        if (checkAllIsNotEmpty(name, surname1, surname2, email, password)) {
            /**Check email is valid*/
            if (email.isEmailValid()) {
                /**Check password are at least 6 characters long*/
                if (password.length > 5) {
                    /**Check selected type user*/
                    if (!typeUser.equals("Undefined")) {
                        // progressbar
                        /**Register user with Firebase Authentication*/
                        auth.createUserWithEmailAndPassword(email, password)
                            .addOnCompleteListener(this) {
                                /**Register user at Firebase Realtime Database */
                                val alumno = Alumno(name, surname1, surname2, email, authorizationToken,
                                    accessToken, refreshToken, urlPhoto, expiresAt)
                                if (typeUser.equals("Alumno")) {
                                    /** Add Alumno Object */
                                    MyFirebaseAPI(this).addStudent(alumno, object : MyCallback{
                                        override fun onAddStudent(b: Boolean) {
                                            if (b){
                                                /**Change to Student empty activity*/
                                                val intent = Intent(this@RegisterActivity, StudentEmptyActivity::class.java)
                                                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
                                                finish()
                                                startActivity(intent)
                                            }else{
                                                /** Show toast*/
                                                Toast.makeText(this@RegisterActivity, "Error al añadir alumno", Toast.LENGTH_SHORT)
                                                    .show()
                                            }

                                        }
                                    })
                                } else {
                                    val teacher = Profesor(name,surname1,surname2,email)
                                    MyFirebaseAPI(this).addTeacher(teacher, object : MyCallback{
                                        override fun onAddTeacher(b: Boolean) {
                                            if (b){
                                                /**Change to Teacher main activity*/
                                                val intent = Intent(this@RegisterActivity, TeacherInitActivity::class.java)
                                                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
                                                finish()
                                                startActivity(intent)
                                            }else{
                                                Toast.makeText(this@RegisterActivity, "Error al añadir profesor", Toast.LENGTH_SHORT)
                                                    .show()
                                            }
                                        }
                                    })
                                }
                                // updateUserInfoAndNextActivity()
                            }
                            .addOnFailureListener {
                                Toast.makeText(this, "Error en la autenticación", Toast.LENGTH_SHORT)
                                    .show()
                            }
                    } else {
                        Toast.makeText(this, "Se debe escoger el tipo de usuario", Toast.LENGTH_SHORT)
                            .show()
                    }
                } else {
                    Toast.makeText(this, "La contraseña debe tener 6 caracteres como mínimo", Toast.LENGTH_SHORT)
                        .show()
                }
            } else {
                Toast.makeText(this, "La dirección de email debe ser válida", Toast.LENGTH_SHORT)
                    .show()
            }
        } else {
            Toast.makeText(this, "Se deben rellenar todos los campos", Toast.LENGTH_SHORT)
                .show()
        }
    }

    private fun updateUserInfoAndNextActivity() {
        val intent = Intent(this, StudentEmptyActivity::class.java)
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
        startActivity(intent)
    }


    private fun String.isEmailValid(): Boolean {
        return !TextUtils.isEmpty(this) && android.util.Patterns.EMAIL_ADDRESS.matcher(this)
            .matches()
    }

    private fun checkAllIsNotEmpty(
            name: String, surname1: String, surname2: String, email: String, password: String): Boolean {
        return name.isNotEmpty() && surname1.isNotEmpty() && surname2.isNotEmpty() && email.isNotEmpty() && password.isNotEmpty()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        // Step 1: Requested authorization code
        if (requestCode == 1) {
            if (resultCode == RESULT_OK) {
                val token: String? = data?.getStringExtra("code")
                authorizationToken = token
                loading = LoadingDialog(this)
                loading.startLoading()
                // Step 2: Requested access token and refresh token
                token?.let {
                    myStrava().linkStravaAccount(it, object : Callback {

                        override fun onFailure(call: Call, e: IOException) {
                            loading.isDismiss()
                        }

                        override fun onResponse(call: Call, response: Response) {
                            if (response.isSuccessful) {
                                val responseStr: String = response.body!!.string()
                                val mapperAll = ObjectMapper()
                                val objData = mapperAll.readTree(responseStr)
                                refreshToken = objData.get("refresh_token").textValue()
                                accessToken = objData.get("access_token").textValue()
                                urlPhoto = objData.get("athlete").get("profile").textValue()
                                expiresAt = objData.get("expires_at").longValue().toString()
                                /**To update UI we need to do it from main thread. As we are in a callback we can use unOnUiThread  */
                                runOnUiThread {
                                    Picasso.get()
                                        .load(urlPhoto)
                                        .transform(CropCircleTransformation())
                                        .placeholder(R.drawable.placeholder)
                                        .error(R.drawable.placeholder)
                                        .into(ivPhoto);
                                }
                                loading.isDismiss()
                            } else {
                                // Request not successful
                                loading.isDismiss()
                            }
                        }
                    })
                }
            }
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when(item.itemId){
            R.id.exit -> {
                val intent = Intent(Intent.ACTION_MAIN)
                intent.addCategory(Intent.CATEGORY_HOME)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                startActivity(intent)

            }
            R.id.logOut -> {
                MyFirebaseAPI(this).logOut()
                val intent = Intent(this, MainActivity::class.java)
                finish()
                startActivity(intent)
            }
            android.R.id.home -> {
                onBackPressed()
            }
        }
        return super.onOptionsItemSelected(item)
    }
}