package com.example.angel.metrosalud

import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import com.example.angel.metrosalud.adapters.SafeClickListener
import com.example.angel.metrosalud.data.Achievement
import com.example.angel.metrosalud.data.Alumno
import com.example.angel.metrosalud.data.Puntos
import com.example.angel.metrosalud.databinding.ActivityStudentEmptyBinding
import com.example.angel.metrosalud.myFirebase.*
import com.example.angel.metrosalud.myStrava.myStrava
import com.example.angel.metrosalud.myUI.LoadingDialog
import com.fasterxml.jackson.databind.ObjectMapper
import com.squareup.picasso.Picasso
import jp.wasabeef.picasso.transformations.CropCircleTransformation
import okhttp3.Call
import okhttp3.Callback
import okhttp3.Response
import java.io.IOException


class StudentEmptyActivity : AppCompatActivity() {
    private lateinit var binding: ActivityStudentEmptyBinding
    private var loading = LoadingDialog(this)
    private var alumno: Alumno? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityStudentEmptyBinding.inflate(layoutInflater)
        setContentView(binding.root)
        supportActionBar?.title = "Mi Perfil"
        //initialise()
        loading.startLoading()
        MyFirebaseAPI(this).getStudent(object : MyCallback {
            override fun onStudent(alu: Alumno) {
                loading.isDismiss()
                alumno = alu
                initialise()
                updateUI(alu)
            }
        })
    }

    private fun initialise() {
        binding.imBtnStrava.visibility = View.GONE
        binding.btnSubmitGroupCode.visibility = View.GONE
        binding.tvName.text = R.string.name_hint.toString()
        binding.tvPoints.text = "0"
        binding.btnSubmitGroupCode.setSafeOnClickListener {
            loading.startLoading()
            MyFirebaseAPI(this).getIdGroup(binding.etGroupCode.text.toString(),
                object : MyCallback {
                    override fun onIdGroup(id: String) {
                        alumno?.idGrupo = id
                        MyFirebaseAPI(
                            this@StudentEmptyActivity).updateIdGroupOnStudent(
                            id, object : MyCallback {
                                override fun onUpdateIdGroupOnStudent(
                                    bool: Boolean) {
                                    if (bool) {
                                        /** Add Points object on Firebase RealTime Database*/
                                        /** First, get StationSize to create object with all null values*/
                                        MyFirebaseAPI(this@StudentEmptyActivity).getStationSize(
                                            id, object : MyCallback{
                                                override fun onStationSize(size: Int) {
                                                    var achievements: MutableList<Achievement> = mutableListOf()
                                                    for ( i in 0 until size){
                                                        achievements.add(Achievement("","","",""))
                                                    }
                                                    val points = Puntos("0",achievements)
                                                    /**Second, add points object*/
                                                    MyFirebaseAPI(this@StudentEmptyActivity).addPoints(
                                                        points, id, object : MyCallback{
                                                            override fun onAddPoints(b: Boolean) {
                                                                if(b){
                                                                    /**Change to activity StudentProfileActivity */
                                                                    val intent = Intent(this@StudentEmptyActivity,
                                                                        StudentProfileActivity::class.java)
                                                                    intent.putExtra("alu", alumno)
                                                                    loading.isDismiss()
                                                                    finish()
                                                                    startActivity(intent)

                                                                }else{
                                                                    loading.isDismiss()
                                                                    Toast.makeText(this@StudentEmptyActivity, "Error generando puntos", Toast.LENGTH_SHORT)
                                                                        .show()
                                                                }
                                                            }
                                                    })
                                                }
                                        })
                                    } else {
                                        loading.isDismiss()
                                        Toast.makeText(this@StudentEmptyActivity, "Error con el código de inscripción", Toast.LENGTH_SHORT)
                                            .show()
                                    }
                                }

                            })
                    }

                })
        }
        binding.imBtnStrava.setSafeOnClickListener {
            val intent = Intent(this, AuthorizeStravaActivity::class.java)
            startActivityForResult(intent, 1)
        }

    }

    private fun updateUI(alu: Alumno) {
        binding.tvName.text = "${alu.nombre} ${alu.apellido1} ${alu.apellido2}"
        Picasso.get()
            .load(alu.urlPhoto)
            .transform(CropCircleTransformation())
            .placeholder(R.drawable.placeholder)
            .error(R.drawable.placeholder)
            .into(binding.ivProfilePhoto);
        if (alu.accessToken == null) {
            binding.imBtnStrava.visibility = View.VISIBLE
        }else{
            binding.btnSubmitGroupCode.visibility = View.VISIBLE
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when(item.itemId){
            R.id.exit -> {
                val intent = Intent(Intent.ACTION_MAIN)
                intent.addCategory(Intent.CATEGORY_HOME)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                startActivity(intent)

            }
            R.id.logOut -> {
                MyFirebaseAPI(this).logOut()
                val intent = Intent(this, MainActivity::class.java)
                finish()
                startActivity(intent)
            }
            android.R.id.home -> {
                onBackPressed()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    private fun View.setSafeOnClickListener(onSafeClick: (View) -> Unit) {
        val safeClickListener = SafeClickListener {
            onSafeClick(it)
        }
        setOnClickListener(safeClickListener)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        // Step 1: Requested authorization code
        if (requestCode == 1) {
            if (resultCode == RESULT_OK) {
                val token: String? = data?.getStringExtra("code")
                loading = LoadingDialog(this)
                loading.startLoading()
                // Step 2: Requested access token and refresh token
                if (token != null) {
                    myStrava().linkStravaAccount(token, object : Callback{
                        override fun onFailure(call: Call, e: IOException) {
                        }

                        override fun onResponse(call: Call, response: Response) {
                            if (response.isSuccessful) {
                                val responseStr: String = response.body!!.string()
                                val mapperAll = ObjectMapper()
                                val objData = mapperAll.readTree(responseStr)
                                alumno?.refreshToken = objData.get("refresh_token").textValue()
                                alumno?.accessToken = objData.get("access_token").textValue()
                                alumno?.urlPhoto = objData.get("athlete").get("profile").textValue()
                                alumno?.expiresAt = objData.get("expires_at").longValue().toString()
                                /**To update UI we need to do it from main thread. As we are in a callback we can use unOnUiThread  */
                                runOnUiThread {
                                    Picasso.get()
                                        .load(alumno?.urlPhoto)
                                        .transform(CropCircleTransformation())
                                        .placeholder(R.drawable.placeholder)
                                        .error(R.drawable.placeholder)
                                        .into(binding.ivProfilePhoto);
                                }
                                /**UPDATE ALUMNO */
                                alumno?.let { it1 ->
                                    MyFirebaseAPI(this@StudentEmptyActivity).updateStudent(it1, object : MyCallback{
                                        override fun onUpdateStudent(b: Boolean) {
                                            if(b){
                                                binding.btnSubmitGroupCode.visibility = View.VISIBLE
                                            }else{
                                                Toast.makeText(this@StudentEmptyActivity, "Se ha producido un error al actualizar los datos del alumno", Toast.LENGTH_SHORT)
                                                    .show()
                                            }
                                            loading.isDismiss()
                                        }
                                    })
                                }
                            } else {
                                // Request not successful
                                loading.isDismiss()
                            }
                        }
                    })
                }
            }
        }
    }
}