package com.example.angel.metrosalud

import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import com.example.angel.metrosalud.adapters.SafeClickListener
import com.example.angel.metrosalud.adapters.StudentProfileStationAdapter
import com.example.angel.metrosalud.data.Achievement
import com.example.angel.metrosalud.data.Alumno
import com.example.angel.metrosalud.data.Parada
import com.example.angel.metrosalud.data.ParadaWithAchievements
import com.example.angel.metrosalud.databinding.ActivityStudentProfileBinding
import com.example.angel.metrosalud.myFirebase.MyCallback
import com.example.angel.metrosalud.myFirebase.MyFirebaseAPI
import com.example.angel.metrosalud.myStrava.myStrava
import com.fasterxml.jackson.databind.ObjectMapper
import com.squareup.picasso.Picasso
import jp.wasabeef.picasso.transformations.CropCircleTransformation
import okhttp3.Call
import okhttp3.Response
import java.io.IOException
import javax.security.auth.callback.Callback

class StudentProfileActivity : AppCompatActivity(), StudentProfileStationAdapter.OnItemClickListener {

    private lateinit var binding: ActivityStudentProfileBinding
    private lateinit var alumno: Alumno

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityStudentProfileBinding.inflate(layoutInflater)
        setContentView(binding.root)
        supportActionBar?.title = "Mi perfil"
        alumno = intent.getSerializableExtra("alu") as Alumno
        initialise()
    }

    override fun onRestart() {
        super.onRestart()
        //Read alumno from Firebase because points could had been changed
        initialise()
    }

    private fun initialise(){
        updateUI(alumno)
        /**Get Station list from Firebase Group Object*/
        MyFirebaseAPI(this@StudentProfileActivity).getStations(alumno.idGrupo, object : MyCallback {
            override fun onStations(stationList: MutableList<Parada>) {
                /**Get completion status from Firebase Points Object and update stationList*/
                alumno.idGrupo?.let { MyFirebaseAPI(this@StudentProfileActivity).getPoints(it,object: MyCallback{
                    override fun onGetPoints(list: MutableList<Achievement>) {
                        var myAchievements = arrayListOf<ParadaWithAchievements>()
                        for (i in 0 until list.size){
                            /**CONSTRUIR UN NUEVO OBJETO Y PASARSELO AL RECYCLER VIEW */
                            myAchievements.add(ParadaWithAchievements(stationList[i].name, stationList[i].description, stationList[i].type,
                                stationList[i].bronze, stationList[i].silver, stationList[i].gold, stationList[i].points,stationList[i].dataIni,
                                stationList[i].dataEnd, list[i].name, list[i].medal, list[i].date, list[i].points))
                        }
                        /**Init recycler view*/
                        initRecycler(myAchievements)
                    }
                }) }
            }
        })

        /** Get group name from Firebase*/
        alumno.idGrupo?.let {
            MyFirebaseAPI(this).getGroupName(it, object : MyCallback{
                override fun onGetGroupName(str: String){
                    binding.tvGroupName.text = str
                }
            })
        }
        /** Get points from Firebase*/
        alumno.idGrupo?.let {
            MyFirebaseAPI(this).getTotalPoints(it, object : MyCallback{
                override fun onGetTotalPoints(str: String){
                    binding.tvPoints.text = str
                }
            })
        }
    }

    private fun initRecycler(list: ArrayList<ParadaWithAchievements>) {
        val llm = LinearLayoutManager(this)
        binding.rvStationStudentProfile.layoutManager = llm
        val adapter = StudentProfileStationAdapter(list, this)
        binding.rvStationStudentProfile.adapter = adapter
        binding.rvStationStudentProfile.setSafeOnClickListener{
            Log.d("PARADA", it.toString())
        }
        val dividerDecoration = DividerItemDecoration(this, llm.orientation)
        binding.rvStationStudentProfile.addItemDecoration(dividerDecoration)
    }


    private fun updateUI(alu: Alumno) {
        binding.tvPoints.text = alu.puntos
        binding.tvName.text = "${alu.nombre} ${alu.apellido1} ${alu.apellido2}"
        Picasso.get()
            .load(alu.urlPhoto)
            .transform(CropCircleTransformation())
            .placeholder(R.drawable.placeholder)
            .error(R.drawable.placeholder)
            .into(binding.ivProfilePhoto);
    }

    override fun onClickItem(item: ParadaWithAchievements, position: Int) {
        /**Read points on that station and pass item and achievement data to next Activity*/
        val intent = Intent(this, AchievementActivity::class.java)
        intent.putExtra("item", item)
        intent.putExtra("alumno", alumno)
        intent.putExtra("index", position)
        intent.putExtra("totalPoints",binding.tvPoints.text.toString().toInt())
        startActivity(intent)

    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        menu?.getItem(2)?.isVisible = true
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when(item.itemId){
            R.id.exit -> {
                val intent = Intent(Intent.ACTION_MAIN)
                intent.addCategory(Intent.CATEGORY_HOME)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                startActivity(intent)

            }
            R.id.logOut -> {
                MyFirebaseAPI(this).logOut()
                val intent = Intent(this, MainActivity::class.java)
                finish()
                startActivity(intent)
            }
            R.id.exitGroup -> {
                //Set -1 on idGroup
                /*MyFirebaseAPI(this).updateIdGroupOnStudent("-1", object : MyCallback {
                    override fun onUpdateIdGroupOnStudent(bool: Boolean) {
                        if (bool){
                            //Set 0 points on Alumno object
                            MyFirebaseAPI(this@StudentProfileActivity).setZeroPointsOnStudent(object : MyCallback{
                                override fun onSetZeroPointsOnStudent(b: Boolean) {
                                    if (b){
                                        // Delete object points
                                        val intent = Intent(this@StudentProfileActivity, StudentEmptyActivity::class.java)
                                        finish()
                                        startActivity(intent)
                                    }
                                }
                            })
                        }
                    }
                })*/
                val newAlumnoValue: Alumno = alumno
                newAlumnoValue.idGrupo = "-1"   // New idGroup value
                newAlumnoValue.puntos = "0"     // New points value
                MyFirebaseAPI(this).updateStudent(newAlumnoValue, object : MyCallback{
                    override fun onUpdateStudent(b: Boolean) {
                        if(b){
                            val intent = Intent(this@StudentProfileActivity, StudentEmptyActivity::class.java)
                            finish()
                            startActivity(intent)
                        }else{
                            Toast.makeText(this@StudentProfileActivity, "Error actualizando datos del alumno", Toast.LENGTH_LONG)
                                .show()
                        }
                    }
                })
            }
        }
        return super.onOptionsItemSelected(item)
    }

    private fun View.setSafeOnClickListener(onSafeClick: (View) -> Unit) {
        val safeClickListener = SafeClickListener {
            onSafeClick(it)
        }
        setOnClickListener(safeClickListener)
    }
}