package com.example.angel.metrosalud

import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem

import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.recyclerview.widget.LinearLayoutManager
import com.example.angel.metrosalud.adapters.GroupAdapter
import com.example.angel.metrosalud.data.Alumno
import com.example.angel.metrosalud.data.Group
import com.example.angel.metrosalud.databinding.ActivityTeacherInitBinding
import com.example.angel.metrosalud.myFirebase.MyCallback
import com.example.angel.metrosalud.myFirebase.MyFirebaseAPI
import com.example.angel.metrosalud.myUI.LoadingDialog

class TeacherInitActivity : AppCompatActivity(), GroupAdapter.OnItemClickListener {

    private lateinit var binding: ActivityTeacherInitBinding
    private lateinit var myGroupsList: ArrayList<Group>
    private lateinit var loading: LoadingDialog


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityTeacherInitBinding.inflate(layoutInflater)
        setContentView(binding.root)
        supportActionBar?.title = "Mis Grupos"
        loading = LoadingDialog(this)
        myGroupsList = arrayListOf<Group>()
        MyFirebaseAPI(this).getGroups(object: MyCallback{
            override fun onGetGroups(list: ArrayList<Group>) {
                if (list != null){
                    myGroupsList.addAll(list.filterNotNull())
                    initRecyclerView()
                }
            }
        })
        initialise()
    }

    private fun initRecyclerView() {
        binding.rvGroup.layoutManager = LinearLayoutManager(this)
        val adapter = GroupAdapter(myGroupsList, this)
        binding.rvGroup.adapter = adapter

    }

    private fun initialise() {
        binding.AddGroupsButton.setOnClickListener {
            val intent = Intent(this,CreateGroupActivity::class.java)
            startActivity(intent)
        }
    }

    public override fun onStart() {
        super.onStart()
        }

    override fun onRemoveClick(position: Int) {
        /** Create confirmation Dialog*/
        val dialog = AlertDialog.Builder(this)
            .setTitle(R.string.dialog_remove_group_title)
            .setMessage(R.string.dialog_remove_group_message)
            .setNegativeButton(R.string.dialog_remove_group_message_negative_btn){ view, _ ->
                Toast.makeText(this, "Operación cancelada", Toast.LENGTH_SHORT).show()
                view.dismiss()
            }
            .setPositiveButton(R.string.dialog_remove_group_message_positive_btn){ view, _ ->
                /** Remove from Firebase Database*/
                val idGroup: String? = myGroupsList[position].id
                if (idGroup != null) {
                    loading.startLoading()
                    MyFirebaseAPI(this).removeGroup(idGroup, object : MyCallback{
                        override fun onRemoveGroup(b: Boolean) {
                            if (b){
                                /**Remove from local variable and Recycler View */
                                myGroupsList.removeAt(position)
                                binding.rvGroup.adapter?.notifyItemRemoved(position)
                                /**Unsubscribe students from group*/
                                MyFirebaseAPI(this@TeacherInitActivity).unsubscribeStudents(idGroup,object: MyCallback{
                                    override fun onUnsubscribeStudents(b: Boolean) {
                                        if(b){

                                        }else{
                                            Toast.makeText(this@TeacherInitActivity, "Se ha producido un error " +
                                                    "eliminando los alumnos apuntudados a su grupo", Toast.LENGTH_LONG)
                                                .show()
                                        }
                                        loading.isDismiss()
                                    }
                                })

                            }else{
                                loading.isDismiss()
                                Toast.makeText(this@TeacherInitActivity, "Error eliminando grupo", Toast.LENGTH_SHORT)
                                    .show()
                            }
                        }
                    })
                }
                view.dismiss()
            }
            .setCancelable(false)
            .create()

        dialog.show()

    }

    override fun onShowGroupClick(position: Int) {
        val idGroup: String? = myGroupsList[position].id
        /**Change Activity */
        val intent = Intent(this,LeaderboardActivity::class.java)
        intent.putExtra("idGroup",idGroup)
        startActivity(intent)
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when(item.itemId){
            R.id.exit -> {
                val intent = Intent(Intent.ACTION_MAIN)
                intent.addCategory(Intent.CATEGORY_HOME)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                startActivity(intent)

            }
            R.id.logOut -> {
                MyFirebaseAPI(this).logOut()
                val intent = Intent(this, MainActivity::class.java)
                finish()
                startActivity(intent)
            }
        }
        return super.onOptionsItemSelected(item)
    }
}
