package com.example.angel.metrosalud.adapters

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.example.angel.metrosalud.R
import com.example.angel.metrosalud.data.Group
import com.example.angel.metrosalud.databinding.ItemGroupBinding

class GroupAdapter(
    private val groupList: ArrayList<Group>, private val listener: OnItemClickListener
) :
    RecyclerView.Adapter<GroupAdapter.GroupHolder>() {


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): GroupHolder {
        val layoutInflater = LayoutInflater.from(parent.context)
        return GroupHolder(layoutInflater.inflate(R.layout.item_group, parent, false))
    }

    override fun getItemCount(): Int = groupList.size

    override fun onBindViewHolder(holder: GroupHolder, position: Int) {
        holder.bind(groupList[position], position)
    }

    inner class GroupHolder(val view: View) : RecyclerView.ViewHolder(view) {
        private val binding = ItemGroupBinding.bind(view)
        fun bind(group: Group, position: Int) {
            binding.tvGroupName.text = group.name
            binding.tvCode.text = group.inscriptionCode
            binding.btnRemoveGroup.tag = position
            binding.btnRemoveGroup.setOnClickListener {
                val selectedItemIndex: Int = adapterPosition
                if (selectedItemIndex != RecyclerView.NO_POSITION) {
                    listener.onRemoveClick(selectedItemIndex)
                }
            }

            binding.tvGroupName.setOnClickListener {
                val selectedItemIndex: Int = adapterPosition
                if (selectedItemIndex != RecyclerView.NO_POSITION) {
                    listener.onShowGroupClick(selectedItemIndex)
                }
            }
        }
    }

    interface OnItemClickListener {
        fun onRemoveClick(position: Int)
        fun onShowGroupClick(position: Int)
    }


}


