package com.example.angel.metrosalud.adapters


import android.graphics.Color
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.example.angel.metrosalud.R
import com.example.angel.metrosalud.data.ParadaWithAchievements
import com.example.angel.metrosalud.databinding.ItemStationStudentBinding


class StudentProfileStationAdapter(
    private val stationList: MutableList<ParadaWithAchievements>,
    private val listener: OnItemClickListener
) :
        RecyclerView.Adapter<StudentProfileStationAdapter.StationHolder>() {


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): StationHolder {
        val layoutInflater = LayoutInflater.from(parent.context)
        return StationHolder(layoutInflater.inflate(R.layout.item_station_student, parent, false))
    }

    override fun getItemCount(): Int = stationList.size

    override fun onBindViewHolder(holder: StationHolder, position: Int) {
        val item = stationList[position]
        holder.bind(item, position)
        holder.itemView.setOnClickListener{listener.onClickItem(item, position)}
        if(position % 2 == 0){
            holder.itemView.setBackgroundColor(Color.rgb(243,243,243))
        }

    }

    inner class StationHolder(val view: View) : RecyclerView.ViewHolder(view) {
        private val binding = ItemStationStudentBinding.bind(view)
        fun bind(station: ParadaWithAchievements, position: Int) {
            binding.tvStationName.text = station.name
            binding.tvStationDescription.text = station.description
            if (station.achievementDate != ""){
                binding.tvDateCompleted.text = station.achievementDate
            }else{

                binding.tvDateCompleted.setText(R.string.Not_completed_text)
            }
            if (station.achievementPoints != ""){
                binding.tvPointsStation.text = station.achievementPoints
            }else{
                binding.tvPointsStation.text = "0"
            }
            if (station.achievementMedal != ""){
                if (station.achievementMedal == "bronze"){
                    binding.ivMedal.setImageResource(android.R.drawable.star_big_on)
                }else{
                    if (station.achievementMedal == "silver"){
                        binding.ivMedal.setImageResource(android.R.drawable.btn_star_big_off)
                    }else{
                        if (station.achievementMedal == "gold"){
                            binding.ivMedal.setImageResource(android.R.drawable.btn_star_big_on)
                        }
                    }
                }

            }else{
                binding.ivMedal.setImageResource(android.R.drawable.star_off)
            }
        }
    }

    interface OnItemClickListener {
        fun onClickItem(item: ParadaWithAchievements, position: Int)
    }
}


